#!/usr/bin/env python

"""
Input

- gff file containing primers
- optional output format

Output

- fa/isPcr file with the primer sequences

Algorithm

- for each line in gff

 - **fa output**:
   if current line has a ``SEQUENCE`` attribute, output it
 - **ispcr output**:
   only pcr primers are of interest, and in equivalent pairs
   on encountering gt-xx put it to dict keyed by ``Parent``
   if there are both entries output and remove from dict

Author: Libor Morkovsky, 2012
"""

# This file is a part of Scrimer.
# See LICENSE.txt for details on licensing.
#    Copyright (C) 2012, 2013 Libor Morkovsky

import sys
import os
import itertools
import string

import pybedtools
    
def main():
    if len(sys.argv) < 2:
        sys.exit('use: %s primer_gff [isPcr|table]' % sys.argv[0])

    # output blat isPcr format?
    out_type = 'fasta'
    if 'isPcr' in sys.argv:
        out_type = 'isPcr'
    if 'table' in sys.argv:
        out_type = 'table'

    pcr_pairs = dict()
    pcr_products = dict()
    # hack fix an error in newer pybedtools
    bt = pybedtools.BedTool(sys.argv[1])
    bt._isbam = False
    for f in bt:
        # fasta output
        if out_type == 'fasta':
            if 'SEQUENCE' in f.attrs:
                print ">", f.name
                print f.attrs['SEQUENCE']
        # isPcr output
        elif out_type == 'isPcr':
            if f.fields[2] != 'pcr-primer': continue
            
            pair = pcr_pairs.setdefault(f.attrs['Parent'], {})
            side = f.name[-1]
            pair[side] = f.attrs['SEQUENCE']
            if(len(pair) == 2):
                print f.name[:-1], pair['F'], pair['R']
                del pcr_pairs[f.attrs['Parent']]
        # descriptive table output
        # output only pcr pair and gt primer, if there is at least one valid
        # gt primer
        # the easiest way is to do this in two passes
        # - first fill a table with info on prc-products
        # - second output only the interesting stuff
        # this could be done in one pass using tabix index
        elif out_type == 'table':
            if f.fields[2] == 'rejected-var': continue

            if f.fields[2] == 'pcr-product':
                key = f.name
            else:
                key = f.attrs['Parent']

            pdata = pcr_products.setdefault(key, {})
            if f.fields[2] == 'pcr-product':
                pdata['product'] = f
            else:
                l = pdata.setdefault(f.fields[2], [])
                l.append(f)

    # output table
    # format:
    # variant-name pcr-f pcr-r gt-f gt-r product-size
    if out_type == 'table':
        fnames = ['ID', 'ref-chromosome', 'pcr-F', 'pcr-R', 'product-size', 'pcr-penalty', 'gt-tag',]
        # dp4 is: reference forward, reference reverse, alternate forward, alternate reverse
        att_names = ['SEQUENCE', 'TM', 'GC_PERCENT', 'PENALTY', 'VAR_mindps', 'VAR_dp4', 'VAR_FQ', 'VAR_MQ',]
        
        # print header
        print "\t".join(fnames + att_names)

        for id in sorted(pcr_products.iterkeys()):
            product = pcr_products[id]
            
            # missing gt primers?
            if 'gt-primer' not in product:
                continue
            
            # no valid gt primers
            if all('PROBLEMS' in f.attrs for f in product['gt-primer']):
                continue

            # get primers with orientations
            pcr = {f.name[-1]:f.attrs['SEQUENCE'] for f in product['pcr-primer']}
            
            # there can be several variants in single pcr product
            # duplicate the pcr product in the table for each possible genotyping primer
            gts = [(f.name[-2:], f) for f in product['gt-primer'] if 'PROBLEMS' not in f.attrs]
            
            for tag, f_gt in gts:
                fields = [id, 
                        product['product'].chrom,
                        pcr['F'], 
                        pcr['R'], 
                        product['product'].attrs['PRODUCT_SIZE'],
                        product['product'].attrs['PENALTY'],
                        tag, 
                        ]

                # add selected attributes
                fields.extend(f_gt.attrs[k] if k in f_gt.attrs else '' for k in att_names)

                print '\t'.join(fields)
            
if __name__ == "__main__": main()



"""
A Python interface to the primer3_core executable.

    TODO: it is not possible to keep a persistent primer3 process
     using subprocess module - communicate() terminates the input
     stream and waits for the process to finish

Author: Libor Morkovsky 2012
"""

# This file is a part of Scrimer.
# See LICENSE.txt for details on licensing.
#    Copyright (C) 2012, 2013 Libor Morkovsky

class BoulderIO:
    """Provides Python interface for ``BoulderIO`` format used by Primer3.
    """

    @classmethod
    def parse(self, string):
        r"""Parse a BoulderIO string ``(KEY=VAL\n)``
        return a list of records, where each record is a dictionary
        end of the string implies a single ``'=\n'`` (record separator).
        """
        record_strings = string.split("=\n")
        return [dict(tuple(line.split("=", 1)) for line in record.split("\n") if len(line) > 3) for record in record_strings if len(record) > 3]

    @classmethod
    def deparse(self, records):
        r"""Accepts a dict or a list of dicts, produces a BoulderIO string ``(KEY=VAL\n)``
        with records separated by ``'=\n'``.
        """
        
        # unify the input, create a list with single element
        if type(records) == dict:
            records = [records]

        return "\n=\n".join("\n".join("=".join(kval) for kval in record.iteritems()) for record in records) + "\n=\n"


class Primer3:
    """Wraps Primer3 executable. `kwargs` are converted to strings and used as default parameters
    for each call of primer3 binary.
    """
    def __init__(self, p3path="primer3_core", **kwargs):
        # store path to primer3
        self.p3path = p3path
        
        # add stringized versions of all kwargs to default args
        self.default_params = {}
        str_kw = dict((key, str(val)) for key, val in kwargs.iteritems())
        self.default_params.update(str_kw)

    def call(self, records):
        """Merge each of the records with `default_params`, the record taking precedence,
        call the ``primer3`` binary, 
        parse the output and return a list of dictionaries,
        ``{RIGHT:[], LEFT:[], PAIR:[], INTERNAL:[]}`` for each input record
        uppercase keys (in the result) are the original names from BoulderIO format,
        lowercase keys have no direct equivalent in primer3 output (``position``, ``other-keys``)
        """

        # merge the defaults with current query
        full_records = [dict(self.default_params.items() + record.items()) for record in records]

        # call primer3
        import subprocess
        self.child = subprocess.Popen([self.p3path, '-strict_tags'], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        out, err = self.child.communicate(BoulderIO.deparse(full_records))
        
        # simple check for errors in stderr
        if len(err):
            raise Exception(err)
        
        results = BoulderIO.parse(out)
        
        # parse the results to {RIGHT:[], LEFT:[], PAIR:[], INTERNAL:[]}
        sides = ['RIGHT', 'LEFT', 'PAIR', 'INTERNAL']
        primers = []
        
        for result in results:
            # primers for current result
            res_primers = dict((side, []) for side in sides)
            used_keys = []
            
            for side in sides:
                nret_key = 'PRIMER_%s_NUM_RETURNED' % side
                nret = int(result.get(nret_key, 0))
                used_keys.append(nret_key)
                
                # extract the values for each single primer and put those to 
                # equivalent key
                for num in xrange(nret):
                    template = 'PRIMER_%s_%d_' % (side, num)
                    primer_keys = filter(lambda k: template in k, result.iterkeys())
                    primer = dict((key[len(template):], result[key]) for key in primer_keys)
                    
                    # extract the position, which itself has no extractible name in BoulderIO
                    # only 'PRIMER_LEFT_0'
                    if side != 'PAIR':
                        pos_key = template[:len(template)-1]
                        primer['position'] = result.get(pos_key, "#error!")
                        used_keys.append(pos_key)
                    
                    # keep track of keys used in current record
                    used_keys.extend(primer_keys)
                    
                    res_primers[side].append(primer)
            
            # store all the unused keys for current result
            res_primers['other-keys'] = dict((key, result[key]) for key in result.iterkeys() if key not in used_keys)
            primers.append(res_primers)
        
        return primers

if __name__ == "__main__":
    print "Running tests"
    
    import textwrap
    
    record = BoulderIO.parse(textwrap.dedent(
    """
    SEQUENCE_ID=example
    SEQUENCE_TEMPLATE=GTAGTCAGTAGACGATGACTACTGACGATGCAGACNACACACACACACACAGCACACAGGTATTAGTGGGCCATTCGATCCCGACCCAAATCGATAGCTACGATGACG
    SEQUENCE_TARGET=37,21
    PRIMER_PICK_INTERNAL_OLIGO=0
    PRIMER_OPT_SIZE=18
    PRIMER_MIN_SIZE=15
    PRIMER_MAX_SIZE=21
    PRIMER_MAX_NS_ACCEPTED=3
    PRIMER_PRODUCT_SIZE_RANGE=50-100
    """))

    record_no_res = BoulderIO.parse(textwrap.dedent(
    """
    SEQUENCE_ID=example
    SEQUENCE_TEMPLATE=GTAGTCAGTAGACNATGACNACTGACGATGCAGACNACACACACACACACAGCACACAGGTATTAGTGGGCCATTCGATCCCGACCCAAATCGATAGCTACGATGACG
    SEQUENCE_TARGET=37,21
    PRIMER_TASK=pick_detection_primers
    PRIMER_PICK_LEFT_PRIMER=1
    PRIMER_PICK_INTERNAL_OLIGO=1
    PRIMER_PICK_RIGHT_PRIMER=1
    PRIMER_OPT_SIZE=18
    PRIMER_MIN_SIZE=15
    PRIMER_MAX_SIZE=21
    PRIMER_MAX_NS_ACCEPTED=1
    PRIMER_PRODUCT_SIZE_RANGE=75-100
    SEQUENCE_INTERNAL_EXCLUDED_REGION=37,21
    """))

    default_params = BoulderIO.parse(textwrap.dedent(
    """
    PRIMER_THERMODYNAMIC_PARAMETERS_PATH=/opt/primer3/bin/primer3_config/
    PRIMER_MAX_NS_ACCEPTED=0
    PRIMER_EXPLAIN_FLAG=1
    """))[0]

    print "Testing BoulderIO, single record:", 
    record_dp = BoulderIO.deparse(record)
    record_reparsed = BoulderIO.parse(record_dp)
    if record == record_reparsed:
        print "OK"
    else:
        print "Failed!"

    print "Testing BoulderIO, two records:", 
    two_records = record + record_no_res
    
    record_dp = BoulderIO.deparse(two_records)
    record_reparsed = BoulderIO.parse(record_dp)
    if two_records == record_reparsed:
        print "OK"
    else:
        print "Failed!"
    
    print "Testing Primer3, single record:", 
    p3 = Primer3(**default_params)

    # test for single record
    res = p3.call(record)
    if res[0]['RIGHT'][0]['SEQUENCE'] == 'GTCGGGATCGAATGGCCC':
        print "OK"
    else:
        print "Failed!"
 
    # test for multiple records
    print "Testing Primer3, two records:", 
    res = p3.call(two_records)

    # second record should produce no results
    if len(res[1]['RIGHT']) == 0:
        print "OK"
    else:
        print "Failed!"
    
    # if no exception occurs, the test should be OK
    print "Tests ran OK"



"""
Implementation of vcf filters for :ref:`pyvcf <PyVCF>` `vcf_filter.py`.

Author: Libor Morkovsky 2012
"""

# This file is a part of Scrimer.
# See LICENSE.txt for details on licensing.
#    Copyright (C) 2012, 2013 Libor Morkovsky

import vcf

class DistinguishingVariants(vcf.Filter):
    """Given a group of samples, choose variants that 
    are not shared with the rest of the samples
    """
    name = 'contrast-samples'

    @classmethod
    def customize_parser(self, parser):
      #TODO: argparse.add_mutually_exclusive_group
      # for --sample-numbers
      parser.add_argument('--sample-names', nargs='+', metavar='sample_id',
                help='Names of samples in the group to contrast against the rest')

    def filter_name(self):
        return self.name

    def __init__(self, args):
      self.group = set(args.sample_names)
      import re
      self.splitter = re.compile('[/|]')


    def __call__(self, record):
      # find the names for rest of the samples
      #TODO: this could be done only once in __init__, if there was access to the reader
      rest = set(sam.sample for sam in record.samples if sam.sample not in self.group)
      
      # create sets of alleles in group and in the rest
      from itertools import chain
      alleles = lambda grp: set(chain.from_iterable(self.splitter.split(record.genotype(sam).gt_bases) for sam in grp))
      
      # if there are any alleles in common, filter the record
      try:
          common = alleles(rest) & alleles(self.group)
      except:
          return 'contrast-exception'

      if len(common): return common




#!/usr/bin/env python

"""
Input

- FASTQ file 
- list of indices (0 based)
 
Output

- FASTQ file without the given sequences 

Author: Libor Morkovsky, 2012
"""

# This file is a part of Scrimer.
# See LICENSE.txt for details on licensing.
#    Copyright (C) 2012, 2013 Libor Morkovsky

import sys

def main():
    # read the kill list
    fkill = open(sys.argv[1])
    killlines = fkill.readlines()
    killseq = frozenset([int(line) for line in killlines])
    fkill.close()

    # go through the input, avoid outputting the 4 line blocks 
    # with given indices
    fseq = open(sys.argv[2])
    linenum = 0
    for line in fseq:
        seqnum = linenum / 4
        linenum += 1  
        if seqnum in killseq:
            continue
        print line.strip()

    fseq.close()

if __name__ == "__main__": main()



#!/usr/bin/env python

"""
Find the most common letter in first n bases of reads in FASTQ file.
Useful for finding and recognizing primer sequences in the reads.
"""

# This file is a part of Scrimer.
# See LICENSE.txt for details on licensing.
#    Copyright (C) 2012, 2013 Libor Morkovsky

from Bio import SeqIO
import sys

def main():
    if len(sys.argv) < 2:
        print >> sys.stderr, "use: %s fastq_file [n-bases=30]" % sys.argv[0]

    file = SeqIO.parse(sys.argv[1], "fastq")

    # stats for first n chars
    nbases = 30
    if len(sys.argv) > 2:
      nbases = int(sys.argv[2])
    d = [dict() for x in range(nbases)]

    for rec in file:
      for i in range(min(len(d), len(str(rec.seq)))):
        letter = str(rec.seq)[i]
        if letter not in d[i]:
          d[i][letter] = 0
          
        d[i][letter] += 1

    # output the stats
    from numpy import var, array
    topseq = ""
    for i in range(len(d)):
      # normalize the histogram to the frequencies
      res = array([float(x)/sum(d[i].values()) for x in d[i].values()])
      # print res
      idx = res.argmax()
      topletter = d[i].keys()[idx]
      print i, ": ", topletter, res[idx], var(res)
      topseq += topletter
      
    print "top sequence:", topseq

if __name__ == '__main__': main()


#! /usr/bin/env python

"""
Input 

- gff file produced by the exon predictor software
- set of bed files containing the features to be transfered
  to the cDNA

Output

- 'inverted' gff file annotating exons in the transcripts
  and containing also all the features from the other input files, 
  if they were overlapping with the predicted exons

Author: Libor Morkovsky, 2012
"""

# This file is a part of Scrimer.
# See LICENSE.txt for details on licensing.
#    Copyright (C) 2012, 2013 Libor Morkovsky

import sys
import itertools
import re

import pybedtools

def main():
    if len(sys.argv) < 2:
      sys.exit('use: %s gff_with_exons [bed_with_fetures] [...]' % sys.argv[0])
      
    # load all feature files as IntervalFiles
    # convert to bed6 if needed
    bedtools = [pybedtools.BedTool(fname) for fname in sys.argv[2:]]
    # fix for some damned bug in pybedtools
    for bt in bedtools:
        bt._isbam = False
    featurefiles = [bedtool.as_intervalfile() if bedtool.field_count() < 12 else bedtool.bed6().as_intervalfile() for bedtool in bedtools]

    gff = pybedtools.BedTool(sys.argv[1])
    if gff.file_type != 'gff':
      sys.exit('supply gff as the exon map file!')

    #
    # gmap output (GFF type), with annotations (GFF attribute):
    #  gene - target id(Name)
    #  mRNA - target id(Name), coverage (Coverage), identity (Identity)
    #  exon - target position (Target)
    #  CDS - target position (Target)
    # sim4db output:
    #  mRNA - target position (split(Name, ':')[1] and split(split(Target, ':')[1])[0]), targetLen, pA, pT, candidate geneRegion 
    #  exon - target position (dtto), Gap, nMatches, intron (bad GFF fomat, can contain unescaped =)
    # 
    # --> to map gene and mRNA blocks from both formats (specifically gmap)
    # we'd have either to fully parse the gff (with the relationships) - better solution
    # or to get a faidx file for the target 'genome' (set of transcripts) 
    # and set the mrna and gene features to contain the whole transcript - easier solution
    # 
    # go through the gff file, outputting the 'inverted' exons
    # and potentially intersecting features from the other files
    #
    """
    GFF
      1: "seqid"
      2: "source"
      3: "type"
      4 & 5: "start" and "end"
      6: "score"
      7: "strand"
      8: "phase"
      9: "attributes"
    """
    rmnum = re.compile('^[0-9]+:')

    for gffeature in gff:
      # as a first approach we transfer only the exons and CDS
      if gffeature.fields[2] != 'exon' and gffeature.fields[2] != 'CDS': continue

      # extract target info  from the attributes
      target_id, target_start, target_end, target_strand = gffeature.attrs['Target'].split()
      
      # sanity check, do not go on if coords are strange
      if int(target_start) > int(target_end): 
        print >> sys.stderr, "target coords weird (%d, %d)" % (int(target_start), int(target_end))
        continue
      
      # remove the leading 'number:' if present (sim4db)
      target_id = rmnum.sub('', target_id)
      gffeature.chrom = rmnum.sub('', gffeature.chrom)
      
      # we do not transfer the parent features now, 
      # so do not break the format and remove the Parent entry
      if 'Parent' in gffeature.attrs: del gffeature.attrs['Parent']
      
      # use the .fields[] to avoid coordinate conversion to bed format (-1 for start)
      gffeature.attrs['Target'] = "%s %s %s %s" % (gffeature.chrom, gffeature.fields[3], gffeature.fields[4], gffeature.strand)
      
      # transfer the source application from original gff to attributes in the output
      gffeature.attrs['source'] = gffeature.fields[1]
      
      # copy the rest of the attribues intact
      gflist = [target_id, 'liftover', gffeature.fields[2], 
        target_start, target_end, gffeature.score, target_strand,
        gffeature.fields[7], str(gffeature.attrs)]
      
      # create the inverted feature
      invgff = pybedtools.create_interval_from_list(gflist)
      print str(invgff).strip()

      # look for annotations overlapping only exons (CDS are contained in exons) 
      if gffeature.fields[2] != 'exon': continue
      
      # find features (consider them exons) in the additional files
      # that intersect with current exon (merge the hits from all files)
      intersecting = itertools.chain.from_iterable([ff.all_hits(gffeature) for ff in featurefiles])
      
      # for each intersecting feature create a clipped feature
      for feature in intersecting:
        
        fs, fe = feature.start, feature.end

        #FIXME: remove the ' ' arg, temporary fix for pybedtools
        at = pybedtools.Attributes(' ');
        
        
        at['Name'] = feature.name
        at['coords'] = gffeature.fields[1]
        
        # this is not necessary if the extension is .gff3 or there is a #version tag in the 
        # beginning of the gff file
        # -
        # because of the igv feature merging, add some uniqness 
        #at['seqid'] = target_id
        
        # left clip
        lclip = gffeature.start - feature.start
        if lclip > 0:
          at['leftClip'] = str(lclip)
          feature.start = int(target_start)
        else:
          feature.start = max(int(target_start) - lclip, 1)
        
        # right clip
        rclip = feature.end - gffeature.end
        if rclip > 0:
          at['rightClip'] = str(rclip)
          feature.end = int(target_end)
        else:
          feature.end = min(int(target_start) + (feature.end - gffeature.start), int(target_end))
        
        #TODO: review the rebasing calculation, still produces start>end
        # quick fix:
        feature.start, feature.end = sorted([feature.start, feature.end])
        
        # sanity check
        # if feature.start < 1:
        #  print >> sys.stderr, str(feature).strip(), target_start, target_end, lclip, rclip, fs, fe
        
        flist = [target_id, 'liftover', 'exon',
          str(feature.start), str(feature.end), str(gffeature.score), feature.strand,
          '.', str(at)]
        
        print str(pybedtools.create_interval_from_list(flist)).strip()

if __name__ == "__main__": main()



#!/usr/bin/env python
"""
Break a fasta file into fragments.

$$$ todo: spread out the fragment starts so that the last fragment ends at the
$$$       .. end of a sequence, if possible

$$$ todo: find runs of N and reset the fragment start position to skip past
$$$       .. such runs

$$$ liborm(2012): fixed to read only the seq name, not the full line
"""

from sys import argv,stdin,exit


def usage(s=None):
    message = """fasta_fragments [options] < fasta_file > fasta_file
  Split a fasta file into overlapping fragments.

  options:
    --fragment=<length>       length of each fragment
                              (default is 100)
    --step=<length>           distance between the start of each fragment
                              (default is 50)"""

    if (s == None): exit (message)
    else:           exit ("%s\n%s" % (s,message))


def main():

    fragmentLength = 100
    stepLength     = 50

    for arg in argv[1:]:
        if (arg.startswith("--fragment=")):
            fragmentLength = int(arg.split("=",1)[1])
        elif (arg.startswith("--step=")):
            stepLength = int(arg.split("=",1)[1])
        elif (arg.startswith("--")):
            usage("can't understand %s" % arg)
        else:
            usage("can't understand %s" % arg)

    allN = "N" * fragmentLength

    # process the sequences

    for (name,seq) in fasta_sequences(stdin):
        seq = seq.upper()
        for ix in xrange(0,len(seq)-fragmentLength,stepLength):
            frag = seq[ix:ix+fragmentLength]
            if (frag == allN): continue
            print ">%s_%d" % (name,ix+1)
            print frag


# fasta_sequences--
#	Read the fasta sequences from a file

def fasta_sequences(f):
    seqName = None
    seqNucs = None

    for line in f:
        line = line.strip()

        if (line.startswith(">")):
            if (seqName != None):
                yield (seqName,"".join(seqNucs))
            seqName = line[1:].strip().split()[0]
            seqNucs = []
        elif (seqName == None):
            assert (False), "first sequence has no header"
        else:
            seqNucs += [line]

    if (seqName != None):
        yield (seqName,"".join(seqNucs))


if __name__ == "__main__": main()




#! /usr/bin/env python

"""Script that parses ``CIGAR`` file 
produced by aligning fragments of contigs to a genome (tested with ``smalt`` output)
and outputs a 'script' for limiting the exon model regions of ``sim4db``.

Input

- output of some read mapper in ``CIGAR`` format

 - all the fragments must be reported by the aligner
 - the fragment names have to be in the same order as the master sequences
 - the fragments must be named like readname_number (like ``fasta_fragments.py`` does)
 - all the hits from one read must follow each other
 
Output

- sim4db 'script'

Algorithm

- load chromosome definition file
- parse the hits:
  - extract read name
  - check if hit is in known chromosome (report error otherwise)
  - for each hit create +-50 KB region clipped to chromosome ends
  - when a read name different from the previous one is encountered, merge all the regions
  - output each contiguos region as a script line

Author: Libor Morkovsky, 2012
"""

# This file is a part of Scrimer.
# See LICENSE.txt for details on licensing.
#    Copyright (C) 2012, 2013 Libor Morkovsky

import sys
import re 

# add a region
def addregion(regs, chr, start, end, reverse):
  # find chromosome + direction combination
  dir = "f" if not reverse else "r"
  
  key = chr + dir
  
  # if this is the first hit of the strand, create a list of regions
  if key not in regs:
    regs[key] = []
  
  # otherwise try to find overlap with some of the existing regions
  regs[key].append((start, end))
  
    
# clip to chromosome length and output the regions  
# merging to overlapping regions on the same chromosome/strand
def output_regions(chromosomes, num, regions):
  """
  from documentation of sim4db:
  A. The input script file format
   
  [-f|-r] -e ESTidx -D GENidx GENlo GENhi
  where:
  cDNAidx        internal index of the cDNA in the input cDNA fasta file
                 (0..#cDNAseqs-1)
  GENidx         internal index of the genomic sequence in the input genome
                 file (0..#GENseqs-1)
  -f             use the cDNA sequence as is
  -r             use the reverse complement of the cDNA sequence
  GENlo, GENhi   begin and end coordinates of the target genomic region;
                 coordinates are 0-based

  For best performance, the script should be sorted by the genomic
  sequence index. 
  """  
  
  for key, ranges in regions.iteritems():
    (seq, dir) = (key[:len(key)-1], key[len(key)-1])
    
    # 
    # merge overlapping
    # 
    ranges.sort()
    newranges = []
    idx = 0
    # go through elements
    while idx < len(ranges):
      # current region
      (start, end) = ranges[idx]
      
      # go through all regions starting after this one
      for nidx in xrange(idx + 1, len(ranges)):
        (nstart, nend) = ranges[nidx]
        # if there is an overlap, try to extend current region
        if nstart <= end:
          end = max(end, nend)
          # continue with merging after this element
          idx = nidx
        # we're in sorted list, there will be no more overlaps
        else:
          break
      
      idx += 1
      newranges.append((start, end))

    for start, end in newranges:
      if start < 0: start = 0
      if end > chromosomes[seq][1]: end = chromosomes[seq][1]
      print "-%s -e %d -D %d %d %d" % (dir, num, chromosomes[seq][0], start, end)

def main():
    if len(sys.argv) != 2:
      exit("use: %s samtools_fai_of_the_genome \n reads cigar from stdin, writes sim4db script to stdout" % sys.argv[0])

    # number of bases to extend on both sides of the hit
    margin = 50000

    # read the fasta index
    fai = open(sys.argv[1])
    chromosomes = dict()
    for num, line in enumerate(fai):
      fields = line.split("\t")
      chromosomes[fields[0]] = (num, int(fields[1]))

    fai.close()

    print >> sys.stderr, len(chromosomes), "sequences loaded from faidx"

    """
     CIGAR format from ssaha2 doc

    [0] in smalt 'cigar' output is preceded by column like 'cigar:S:56'

    [1] query_name  Name of the query sequence from the input file. 
    [2] query_start  Start of the alignment in the query. 
    [3] query_end  End of the alignment in the query. 
    [4] query_strand  Query orientation (+/-). 
    [5] subject_name  Name of the subject sequence from the input file. 
    [6] subject_start Start of the alignment in the subject. 
    [7] subject_end  End of the alignment in the subject. 
    [8] subject_strand  Subject orientation (always + for SSAHA2). 
    [9] score  Smith-Waterman score obtained from the cross_match alignment.
    [10:] series of <operation, length> pairs where operation is one of match, insertion or deletion
    """
    old_seqname = ""
    seqnum = 0
    regions = dict()
    # loop through the input
    for nline, line in enumerate(sys.stdin):
      fields = line.split()
      seqname = re.sub("_[0-9]+$", "", fields[1])
      
      # if new query sequence encountered, output the regions
      if seqname != old_seqname and nline != 0:
        if len(regions):
            output_regions(chromosomes, seqnum, regions)
            regions = dict()

        seqnum += 1
      
      # store the seqname for next loop
      old_seqname = seqname
      
      # sanity check -- if we're matching against the correct genome
      if fields[5] not in chromosomes:
        # a special case for 'no hit'
        if fields[5] != '*':
            print >> sys.stderr, "hit %d was matched to sequence not in faidx/genome (%s)" % (nline, fields[5])
        continue
      
      # create a region for current hit
      # subj_name, start, end, reverse?
      addregion(regions, fields[5], int(fields[6]) - margin, int(fields[7]) + margin, fields[4] != fields[8])

    # output the ranges for last sequence
    output_regions(chromosomes, seqnum, regions)

if __name__ == '__main__': main()



#!/usr/bin/env python

"""
Input

- reference sequence (fasta with samtools fai index)
- annotations (gff3, has to contain exon entries)
- filtered variants (vcf, primers are designed for variants with PASS)

Output

- PCR and genotyping primers selected using primer3 (gff3)

Algorithm

- there is only a few selected variants, so the least amount of work
  will be to do the work only for variants
- for each of the selected variants

  - request exons
  - apply the technical constraints 
    (minimal primer length of 20 from the edge of an exon)
  - patch exon sequence to mark positions of known variants
  - find suitable genotyping primers
  - design PCR primers to flank the (usable) genotyping primers

Author: Libor Morkovsky, 2012, 2014
"""

# This file is a part of Scrimer.
# See LICENSE.txt for details on licensing.
#    Copyright (C) 2012, 2013 Libor Morkovsky

import sys
import os
import itertools
import argparse

import pysam
import vcf
import pybedtools

from scrimer import primer3_connector

min_gt_primer_len = 20
pref_gt_primer_len = 22
max_gt_primer_len = 28

# get location of config dir from environment
if 'PRIMER3_CONFIG' in os.environ:
    PRIMER3_CONFIG = os.environ['PRIMER3_CONFIG']
else:
    PRIMER3_CONFIG = '/opt/primer3/bin/primer3_config/'

def patched_sequence(seq, seq_start, variants):
    """Returns sequence with all variant sites replaced by Ns
    a functional approach forced by python immutable strings
    seq - sequence string
    seq_start - start of sequence in refernce for variants
    variants - [vcf._Record]
    """
    # consider each variable site only once
    uniq_positions = set((var.POS - 1 - seq_start) for var in variants)
    
    # break the input seq apart, leaving out the variant sites
    ends = sorted(uniq_positions)
    begins = [0] + [pos + 1 for pos in ends]
    ends.append(len(seq))
    fragments = (seq[b:e] for (b, e) in zip(begins, ends))
    
    # substite the variant sites with Ns
    return 'N'.join(fragments)

#TODO: consider PRIMER_LOWERCASE_MASKING=1
def find_pcr_primers(pseq, target):
    """call primer3 executable, return the results
    """
    def_params = {
    'PRIMER_THERMODYNAMIC_PARAMETERS_PATH': PRIMER3_CONFIG,
    'PRIMER_MAX_NS_ACCEPTED':'0',
    }
    
    p3 = primer3_connector.Primer3(**def_params)

    rec_params = {
    'SEQUENCE_ID':'pick_pcr_primers_task',
    'SEQUENCE_TEMPLATE':pseq,
    'SEQUENCE_TARGET': "%d,%d" % (target[0], target[1]),
    'PRIMER_OPT_SIZE':'19',
    'PRIMER_MIN_SIZE':'17',
    'PRIMER_MAX_SIZE':'25',
    'PRIMER_PRODUCT_SIZE_RANGE':'70-300',
    'PRIMER_LOWERCASE_MASKING':'1',
    }
    
    return p3.call([rec_params])

def find_gt_primers(pseq, target):
    """call primer3 executable, return the results
    simulating pick_discriminating_primers with FORCE_*_END
    to avoid off-by-one error present in primer3
    """
    def_params = {
    'PRIMER_THERMODYNAMIC_PARAMETERS_PATH': PRIMER3_CONFIG,
    'PRIMER_MAX_NS_ACCEPTED':'0',
    }
    
    p3 = primer3_connector.Primer3(**def_params)

    rec_params = {
    'PRIMER_TASK':'generic',
    'SEQUENCE_ID':'pick_gt_primers_task',
    'SEQUENCE_TEMPLATE': pseq,
    'SEQUENCE_TARGET': "%d,1" % target,
    'SEQUENCE_FORCE_LEFT_END': str(target - 1),
    'SEQUENCE_FORCE_RIGHT_END': str(target + 1),
    'PRIMER_OPT_SIZE': str(pref_gt_primer_len),
    'PRIMER_MIN_SIZE': str(min_gt_primer_len),
    'PRIMER_MAX_SIZE': str(max_gt_primer_len),
    'PRIMER_PRODUCT_SIZE_RANGE':'%d-%d' % (2*min_gt_primer_len + 1, 2*max_gt_primer_len + 1),
    'PRIMER_MIN_TM':'50',
    'PRIMER_OPT_TM':'55',
    'PRIMER_NUM_RETURN':'1',
    # pick the primers even if there is no decent pair, we don't need pairs
    'PRIMER_PICK_ANYWAY': '1', 
    # pick a thermodynamically worse primer, but avoid Ns at all costs
    'PRIMER_WT_NUM_NS': '100', 
    }
    
    return p3.call([rec_params])

def primer_to_gff(name, primer, tag, seq_name, seq_start, strand, **kwargs):
    """Create a gff feature from 
    partially parsed primer3 results.
    """
    pos, len = map(int, primer['position'].split(','))
    
    # transfer the calculated values to attributes
    # skip the fields used elsewhere in gff
    at = pybedtools.Attributes(' ')
    for k, v in primer.iteritems():
        if k == 'position': continue
        at[k] = v.replace(';', '%3B')
        
    at['ID'] = name
    
    # pass all optional params to attributes
    at.update({k:str(v) for k, v in kwargs.iteritems()})
    
    # primer3 provides the coordinates of right primer with the 
    # pos pointing to the last base
    if strand == '-':
        start = seq_start + pos - len + 2
        end = seq_start + pos + 1
    else:
        start = seq_start + pos + 1
        end = seq_start + pos + len

    gflist = [seq_name, 'design-primers', tag, 
        str(start), str(end),
        primer['PENALTY'], strand, '.', str(at)]

    return pybedtools.create_interval_from_list(gflist)

def get_flanking(seq, pos, size):
    """Get sequence flanking given position
    add Ns if there is no input data (ends of input sequence)
    """
    left = seq[max(pos - size, 0):pos]
    right = seq[pos + 1:pos + size + 1]
    return (left.rjust(size, 'N'), right.ljust(size, 'N'))

def reverse_complement(seq):
    """Return a DNA reverse complement
    """
    compl = string.maketrans('ACGTNacgtn', 'TGCANtgcan')
    return seq.translate(compl)[::-1]

def primers_for_var(genome, annotations, variants_random_access, var):
    """In ideal case this returns list of gff features representing
    - a pair of pcr primers
    - pcr product (carries the thermodynamic penalties for using 
        the combination of current pcr primers)
    - two genotyping primers
    or a feature carrying a Note about the error
    """
    
    def rejected_var_feature(reason):
        """ Output a dummy feature holding the 
        reason for rejection of selected variant in 
        the design process.
        """
        at = pybedtools.Attributes(' ')
        at['Note'] = reason
        at['color'] = "#bb0000"
        
        gflist = [var.CHROM, 'design-primers', 'rejected-var', 
            str(var.POS - max_gt_primer_len), str(var.POS + max_gt_primer_len),
            '0', '+', '.', str(at)]
            
        return pybedtools.create_interval_from_list(gflist)

    # container for the output
    result = []

    var_features = annotations.tabix_intervals(pybedtools.Interval(var.CHROM, var.POS, var.POS + 1))
    var_exons = [f for f in var_features if f.fields[2] == 'exon']
    

    if len(var_exons) == 0: 
        return [rejected_var_feature('no predicted exons')]
    
    # get a minimal predicted exon
    # (trying to be conservative)
    # consider only predicted, not transferred exons (source in attrs)
    ex_predicted = [f for f in var_exons if 'source' in f.attrs]
    ex_start = max(f.start for f in ex_predicted)
    ex_end = min(f.end for f in ex_predicted)
    min_exon = pybedtools.Interval(var.CHROM, ex_start, ex_end)
    
    # check if the exon has a sane size (> 70)
    if min_exon.length < 70: 
        return [rejected_var_feature('minimal exon too short for PCR')]

    # check if the variant is in position that permits PCR amplification
    # (more than 20 bases from both ends)
    # this implies a check if the variant is still inside of the minimal exon
    distances = (var.POS - min_exon.start, min_exon.end - var.POS)
    if min(distances) < 20: 
        return [rejected_var_feature('variant too close to boundary or outside of minimal exon')]
    
    # get all variants for the minimal exon
    exon_variants = variants_random_access.fetch(var.CHROM, min_exon.start, min_exon.end)
    
    # get sequence for the minimal exon
    # and patch all the variants with Ns
    seq = genome.fetch(min_exon.chrom, min_exon.start, min_exon.end)
    pseq = patched_sequence(seq, min_exon.start, exon_variants)
    
    # check if there is at least 20 fixed bases on either side
    # of the target variant because of the genotyping primer
    # (this could be faster exit path than a call to primer3)
    gt_primer_seqs = get_flanking(pseq, var.POS - 1 - min_exon.start, max_gt_primer_len)
    # attach 'N' to end of each primer, so we get the full len
    # instead of -1, when there is no N in the original primer
    max_gt_lens = [(s + 'N').find('N') for s in  [gt_primer_seqs[0][::-1], gt_primer_seqs[1]]]
    if all(x < min_gt_primer_len for x in max_gt_lens): 
        return [rejected_var_feature('no possible genotyping primer longer than %d bp' % min_gt_primer_len)]

    
    # call primer3 to find suitable genotyping primers
    gt_primers = find_gt_primers(pseq, var.POS - 1 - min_exon.start)
    if all(len(p) == 0 for p in [gt_primers[0]['LEFT'], gt_primers[0]['RIGHT']]):
        return [rejected_var_feature('no good genotyping primer found by primer3')]

    # call primer3 to design PCR primers
    # mark the region with suitable genotyping primers as a target
    pcr_min = pcr_max = var.POS - 1 - min_exon.start
    if len(gt_primers[0]['LEFT']):
        pcr_min = int(gt_primers[0]['LEFT'][0]['position'].split(',')[0])
    if len(gt_primers[0]['RIGHT']):
        # primer3 uses coordinates of the 5' end
        pcr_max = int(gt_primers[0]['RIGHT'][0]['position'].split(',')[0])
        
    
    # find primers in the sequence patched with Ns
    primers = find_pcr_primers(pseq, (pcr_min, pcr_max - pcr_min))
    
    # if something was found, output the first pair
    # (should be the best one)
    if len(primers[0]['PAIR']):
        # calculate the 'position' entry PAIR, so it is the same as in LEFT/RIGHT entries and can be used by primer_to_gff
        # to represent the whole-product
        primers[0]['PAIR'][0]['position'] = primers[0]['LEFT'][0]['position'].split(',')[0] + ',' + primers[0]['PAIR'][0]['PRODUCT_SIZE']
        result.append(primer_to_gff('pcr-product', primers[0]['PAIR'][0], 'pcr-product', var.CHROM, min_exon.start, '+'))
        result.append(primer_to_gff('pcr-left', primers[0]['LEFT'][0],  'pcr-primer', var.CHROM, min_exon.start, '+'))
        result.append(primer_to_gff('pcr-right', primers[0]['RIGHT'][0], 'pcr-primer', var.CHROM, min_exon.start, '-'))
    else:
        return [rejected_var_feature('no suitable PCR primers found')]
    
    # decorate all genotyping primers with basic statistics about the variant being genotyped
    
    vkeys = ['FQ', 'MQ']
    more = {('VAR_%s' % k):var.INFO[k] for k in vkeys if k in var.INFO}
    more['VAR_mindps'] = min(sam['DP'] for sam in var.samples)
    if 'DP4' in var.INFO:
        more['VAR_dp4'] = ','.join(map(str, var.INFO['DP4']))

    if len(gt_primers[0]['LEFT']):
        color = '#bb0000' if 'PROBLEMS' in gt_primers[0]['LEFT'][0] else '#00bb00'
        result.append(primer_to_gff('gt-left', gt_primers[0]['LEFT'][0], 'gt-primer', var.CHROM, min_exon.start, '+', 
            color=color, **more))
    if len(gt_primers[0]['RIGHT']):
        color = '#bb0000' if 'PROBLEMS' in gt_primers[0]['RIGHT'][0] else '#00bb00'
        more = {('VAR_%s' % k):var.INFO[k] for k in vkeys if k in var.INFO}
        result.append(primer_to_gff('gt-right', gt_primers[0]['RIGHT'][0], 'gt-primer', var.CHROM, min_exon.start, '-', 
            color=color, **more))
            
    return result

def argparser():
    args = argparse.ArgumentParser(description="Design primers using PRIMER3.")

    args.add_argument("--primer-min", type=int, default=20,
        help="Minimal length of the genotyping primer (default: 20).")
    args.add_argument("--primer-pref", type=int, default=22,
        help="Preferred length of the genotyping primer (default: 22).")
    args.add_argument("--primer-max", type=int, default=28,
        help="Minimal length of the genotyping primer (default: 28).")

    #args.add_argument("--annot-type", default="exon",
    #    help="Annotatoin type that should represent contiguous sequence in DNA (default: exon).")

    args.add_argument("genome",
        help="Fasta file with associated .fai index (use samtools fai to index).")
    args.add_argument("annots",
        help="GFF file with associated .tbi (use tabix to index).")
    args.add_argument("variants",
        help="VCF file with selected variants.")

    return args

def crc(line):
    return zlib.crc32(line)
    
def main():

    parser = argparser()
    args = parser.parse_args() 

    genome = pysam.Fastafile(args.genome)
    annotations = pybedtools.BedTool(args.annots)
    variants = vcf.Reader(filename=args.variants)
    # open the variants once more, so the ifilter is not broken by .fetch
    var_fetcher = vcf.Reader(filename=args.variants)

    # the most commonly changed value will be 
    # preferred length, fix the min/max if pref
    # is not in the default range
    if args.primer_pref < args.primer_min:
        args.primer_min = args.primer_pref
    if args.primer_pref > args.primer_max:
        args.primer_max = args.primer_pref
        
    min_gt_primer_len = args.primer_min
    pref_gt_primer_len = args.primer_pref
    max_gt_primer_len = args.primer_max

    # locus can be either exon or mrna, for now we consider mrna a locus
    # locus_feature_type = args.annot_type
    
    seen_loci = set()

    # by-locus naming convention implies following algo
    # (almost single pass, keeping only the names of loci already seen 
    #  in memory)
    # algorithm:
    # get a chosen variant
    # find locus feature for chosen variant
    # if locus already seen, get next variant
    # find all other chosen variants for locus
    # for each variant in locus design primers
    #  if designed primers overlap with others in locus, deduplicate
    
    # use iterator filter, so the variants are streamed
    for var in itertools.ifilter(lambda v: not v.FILTER, variants):
        # get a locus feature for the var
        var_features = annotations.tabix_intervals(pybedtools.Interval(var.CHROM, var.POS, var.POS + 1))
        # var_loci = [f for f in var_features if f.fields[2] == locus_feature_type]
        var_loci = [f for f in var_features if f.fields[2] == 'mRNA']
        
        # var is not in any known feature
        if len(var_loci) == 0:
            continue

        # quick check for now
        if len(var_loci) > 1:
            print >> sys.stderr, 'too many locus features for %s: %s' % (str(var), str(var_loci))
            continue
        
        locus = var_loci[0]
        
        # all variants from this locus are already done
        if locus.name in seen_loci:
            continue
        else:
            seen_loci.add(locus.name)

        # process all variants in current locus
        # set the names and parenting structure
        locus_variants = [v for v in var_fetcher.fetch(var.CHROM, locus.start, locus.end)]
        locus_primers = []
        pcr_id = 0
        locus_out_name = "LU%04d" % len(seen_loci)
        
        # summary statistics for whole locus
        # used to decorate all products
        pass_variants = sum(1 for var in locus_variants if not var.FILTER)
        
        for lvar_num, lvar in enumerate(itertools.ifilter(lambda v: not v.FILTER, locus_variants)):
            # create a convinient dict from the max-5 feature result
            curr_primers = {f.name:f for f in primers_for_var(genome, annotations, var_fetcher, lvar)}

            # if there was an error designing the primers
            # output the error indicating features
            if 'pcr-product' not in curr_primers:
                map(lambda f: sys.stdout.write(str(f)), curr_primers.itervalues())
                continue

            def add_feature(f, name, parent):
                f.name = name
                f.attrs['Parent'] = parent
                locus_primers.append(f)

            # try to find the same pcr product
            try:
                # if the position of the primers is equal, the sequences are 
                # also equal, so the termodynamic parameters should be the same as well
                # so we can use the Interval() '==' operator, which ignores gff attributes
                # the == operator raises error when the intervals contain each other
                def intervals_eq(a, b):
                    try:
                        return a == b
                    # when intervals contain each other, they're not equal
                    except NotImplementedError:
                        return False
            
                # find function with custom comparator
                def find(seq, item, _cmp):
                    for idx, i in enumerate(seq):
                        if _cmp(i, item): return idx
                    raise ValueError('item not found')

                idx = find(locus_primers, curr_primers['pcr-product'], intervals_eq)
                parent_pcr = locus_primers[idx]
            # new pcr product
            # add the product and pcr primers
            except ValueError:
                pcr_id += 1
                parent_pcr = curr_primers['pcr-product']
                parent_pcr.name = "%s-%d" % (locus_out_name, pcr_id)
                locus_primers.append(curr_primers['pcr-product'])
                
                # decorate the PCR product with reference genome transcript names 
                # and some statistics
                ref_names = set(e.attrs['Name'] for e in var_features if 'coords' in e.attrs)
                if len(ref_names): parent_pcr.attrs['ref_names'] = '|'.join(ref_names)
                parent_pcr.attrs['variants_in_locus'] = str(pass_variants)

                # add pcr primers
                add_feature(curr_primers['pcr-left'], "%sF" % parent_pcr.name, parent_pcr.name)
                add_feature(curr_primers['pcr-right'], "%sR" % parent_pcr.name, parent_pcr.name)

            # add genotyping primers
            if 'gt-left' in curr_primers:
                add_feature(curr_primers['gt-left'], "%s-%dF" % (parent_pcr.name, lvar_num + 1), parent_pcr.name)
            if 'gt-right' in curr_primers:
                add_feature(curr_primers['gt-right'], "%s-%dR" % (parent_pcr.name, lvar_num + 1), parent_pcr.name)
        
        # output all primers for current locus
        map(lambda f: sys.stdout.write(str(f)), locus_primers)

if __name__ == "__main__": main()



#!/usr/bin/env python

try:
    from setuptools import setup
    have_setuptools = True
except ImportError:
    from distutils.core import setup
    have_setuptools = False

import glob

with open('README') as f:
    long_description = f.read()

with open('LICENSE.txt') as f:
    license = f.read()

setup(name='scrimer',
      version='1.1',
      description='Primer designing pipeline',
      author='Libor Morkovsky',
      author_email='morkovsk@natur.cuni.cz',
      url='https://github.com/libor-m/scrimer',
      py_modules=[f.replace('.py', '') for f in glob.glob('scrimer/*.py')],
      scripts=glob.glob('scripts/*.py'),
      # http://stackoverflow.com/questions/6947988/when-to-use-pip-requirements-file-versus-install-requires-in-setup-py
      #install_requires=['PyVCF (>=0.6)', 'pybedtools (>=0.6)', 'pysam (>=0.7)'],
      install_requires=['PyVCF', 'pybedtools', 'pysam'],
      # requires=['PyVCF (>=0.6)', 'pybedtools (>=0.6)', 'pysam (>=0.7)'],
      data_files=[('demo', glob.glob('demo/*'))],
      long_description=long_description,
      # https://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Console',
          'Intended Audience :: Science/Research',
          'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',
          'Natural Language :: English',
          'Operating System :: POSIX',
          'Operating System :: POSIX :: Linux',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2.7',
          'Topic :: Scientific/Engineering :: Bio-Informatics',
      ],
      license=license,
      entry_points="""
      [vcf.filters]
      contrast-samples = scrimer.pyvcf_filters:DistinguishingVariants
      """,
     )



#!/usr/bin/env python
"""Filter a FASTA, FASTQ or SSF file with IDs from a tabular file.

Takes six command line options, tabular filename, ID column numbers (comma
separated list using one based counting), input filename, input type (e.g.
FASTA or SFF) and two output filenames (for records with and without the
given IDs, same format as input sequence file).

If either output filename is just a minus sign, that file is not created.
This is intended to allow output for just the matched (or just the non-matched)
records.

When filtering an SFF file, any Roche XML manifest in the input file is
preserved in both output files.

Note in the default NCBI BLAST+ tabular output, the query sequence ID is
in column one, and the ID of the match from the database is in column two.
Here sensible values for the column numbers would therefore be "1" or "2".

This tool is a short Python script which requires Biopython 1.54 or later
for SFF file support. If you use this tool in scientific work leading to a
publication, please cite the Biopython application note:

Cock et al 2009. Biopython: freely available Python tools for computational
molecular biology and bioinformatics. Bioinformatics 25(11) 1422-3.
http://dx.doi.org/10.1093/bioinformatics/btp163 pmid:19304878.

This script is copyright 2010 by Peter Cock, SCRI, UK. All rights reserved.
See accompanying text file for licence details (MIT/BSD style).

This is version 0.0.1 of the script.
"""
import sys

#Dan Blankenberg

class fastaSequence( object ):
    def __init__( self ):
        self.identifier = None
        self.sequence = '' #holds raw sequence string: no whitespace
    def __len__( self ):
        return len( self.sequence )
    def __str__( self ):
        return "%s\n%s\n" % ( self.identifier, self.sequence )

class fastaReader( object ):
    def __init__( self, fh ):
        self.file = fh
    def close( self ):
        return self.file.close()
    def next( self ):
        line = self.file.readline()
        #remove header comment lines
        while line and line.startswith( '#' ):
            line = self.file.readline()
        if not line:
            raise StopIteration 
        assert line.startswith( '>' ), "FASTA headers must start with >"
        rval = fastaSequence()
        rval.identifier = line.strip()
        offset = self.file.tell()
        while True:
            line = self.file.readline()
            if not line or line.startswith( '>' ):
                if line:
                    self.file.seek( offset ) #this causes sequence id lines to be read twice, once to determine previous sequence end and again when getting actual sequence; can we cache this to prevent it from being re-read?
                return rval
            #454 qual test data that was used has decimal scores that don't have trailing spaces 
            #so we'll need to parse and build these sequences not based upon de facto standards
            #i.e. in a less than ideal fashion
            line = line.rstrip()
            if ' ' in rval.sequence or ' ' in line:
                rval.sequence = "%s%s " % ( rval.sequence, line )
            else:
                rval.sequence += line
            offset = self.file.tell()
    def __iter__( self ):
        while True:
            yield self.next()

class fastaWriter( object ):
    def __init__( self, fh ):
        self.file = fh
    def write( self, fastq_read ):
        #this will include color space adapter base if applicable
        self.file.write( ">%s\n%s\n" % ( fastq_read.identifier[1:], fastq_read.sequence ) )
    def close( self ):
        return self.file.close() 

def stop_err(msg, err=1):
    sys.stderr.write(msg.rstrip() + "\n")
    sys.exit(err)

def main():
    #Parse Command Line
    try:
        tabular_file, cols_arg, in_file, seq_format, out_positive_file, out_negative_file = sys.argv[1:]
    except ValueError:
        stop_err("Expected six arguments, got %i:\n%s" % (len(sys.argv)-1, " ".join(sys.argv)))
    try:
        columns = [int(arg)-1 for arg in cols_arg.split(",")]
    except ValueError:
        stop_err("Expected list of columns (comma separated integers), got %s" % cols_arg)

    if out_positive_file == "-" and out_negative_file == "-":
        stop_err("Neither output file requested")


    #Read tabular file and record all specified identifiers
    ids = set()
    handle = open(tabular_file, "rU")
    if len(columns)>1:
        #General case of many columns
        for line in handle:
            if line.startswith("#"):
                #Ignore comments
                continue
            parts = line.rstrip("\n").split("\t")
            for col in columns:
                ids.add(parts[col])
        print "Using %i IDs from %i columns of tabular file" % (len(ids), len(columns))
    else:
        #Single column, special case speed up
        col = columns[0]
        for line in handle:
            if not line.startswith("#"):
                ids.add(line.rstrip("\n").split("\t")[col])
        print "Using %i IDs from tabular file" % (len(ids))
    handle.close()


    if seq_format.lower()=="sff":
        #Now write filtered SFF file based on IDs from BLAST file
        try:
            from Bio.SeqIO.SffIO import SffIterator, SffWriter
        except ImportError:
            stop_err("Requires Biopython 1.54 or later")

        try:
            from Bio.SeqIO.SffIO import ReadRocheXmlManifest
        except ImportError:
            #Prior to Biopython 1.56 this was a private function
            from Bio.SeqIO.SffIO import _sff_read_roche_index_xml as ReadRocheXmlManifest
        in_handle = open(in_file, "rb") #must be binary mode!
        try:
            manifest = ReadRocheXmlManifest(in_handle)
        except ValueError:
            manifest = None
        #This makes two passes though the SFF file with isn't so efficient,
        #but this makes the code simple.
        if out_positive_file != "-":
            out_handle = open(out_positive_file, "wb")
            writer = SffWriter(out_handle, xml=manifest)
            in_handle.seek(0) #start again after getting manifest
            pos_count = writer.write_file(rec for rec in SffIterator(in_handle) if rec.id in ids)
            out_handle.close()
        if out_negative_file != "-":
            out_handle = open(out_negative_file, "wb")
            writer = SffWriter(out_handle, xml=manifest)
            in_handle.seek(0) #start again
            neg_count = writer.write_file(rec for rec in SffIterator(in_handle) if rec.id not in ids)
            out_handle.close()
        #And we're done
        in_handle.close()
        #At the time of writing, Galaxy doesn't show SFF file read counts,
        #so it is useful to put them in stdout and thus shown in job info.
        if out_positive_file != "-" and out_negative_file != "-":
            print "%i with and %i without specified IDs" % (pos_count, neg_count)
        elif out_positive_file != "-":
            print "%i with specified IDs" % pos_count
        elif out_negative_file != "-":
            print "%i without specified IDs" % neg_count
    elif seq_format.lower()=="fasta":
        #Write filtered FASTA file based on IDs from tabular file
        reader = fastaReader(open(in_file, "rU"))
        if out_positive_file != "-" and out_negative_file != "-":
            print "Generating two FASTA files"
            positive_writer = fastaWriter(open(out_positive_file, "w"))
            negative_writer = fastaWriter(open(out_negative_file, "w"))
            for record in reader:
                #The [1:] is because the fastaReader leaves the > on the identifer.
                if record.identifier and record.identifier.split()[0][1:] in ids:
                    positive_writer.write(record)
                else:
                    negative_writer.write(record)
            positive_writer.close()
            negative_writer.close()
        elif out_positive_file != "-":
            print "Generating matching FASTA file"
            positive_writer = fastaWriter(open(out_positive_file, "w"))
            for record in reader:
                #The [1:] is because the fastaReader leaves the > on the identifer.
                if record.identifier and record.identifier.split()[0][1:] in ids:
                    positive_writer.write(record)
            positive_writer.close()
        elif out_negative_file != "-":
            print "Generating non-matching FASTA file"
            negative_writer = fastaWriter(open(out_negative_file, "w"))
            for record in reader:
                #The [1:] is because the fastaReader leaves the > on the identifer.
                if not record.identifier or record.identifier.split()[0][1:] not in ids:
                    negative_writer.write(record)
            negative_writer.close()
    elif seq_format.lower().startswith("fastq"):
        #Write filtered FASTQ file based on IDs from tabular file
        from galaxy_utils.sequence.fastq import fastqReader, fastqWriter
        reader = fastqReader(open(in_file, "rU"))
        if out_positive_file != "-" and out_negative_file != "-":
            print "Generating two FASTQ files"
            positive_writer = fastqWriter(open(out_positive_file, "w"))
            negative_writer = fastqWriter(open(out_negative_file, "w"))
            for record in reader:
                #The [1:] is because the fastaReader leaves the @ on the identifer.
                if record.identifier and record.identifier.split()[0][1:] in ids:
                    positive_writer.write(record)
                else:
                    negative_writer.write(record)
            positive_writer.close()
            negative_writer.close()
        elif out_positive_file != "-":
            print "Generating matching FASTQ file"
            positive_writer = fastqWriter(open(out_positive_file, "w"))
            for record in reader:
                #The [1:] is because the fastaReader leaves the @ on the identifer.
                if record.identifier and record.identifier.split()[0][1:] in ids:
                    positive_writer.write(record)
            positive_writer.close()
        elif out_negative_file != "-":
            print "Generating non-matching FASTQ file"
            negative_writer = fastqWriter(open(out_negative_file, "w"))
            for record in reader:
                #The [1:] is because the fastaReader leaves the @ on the identifer.
                if not record.identifier or record.identifier.split()[0][1:] not in ids:
                    negative_writer.write(record)
            negative_writer.close()
    else:
        stop_err("Unsupported file type %r" % seq_format)

if __name__ == "__main__": main()



#! /usr/bin/env python

"""
Input

- fasta file with the original sequences
- set of gff files with exon features having the Target attribute 
  (product of ``liftover.py``)
- other gff/bed files to remap to the genome

Output 

- fasta with the input sequences pasted with 100 Ns

  - unambiguos sequences assigned to 'chromosomes' in the order 
    of the template genome (that was used to generate the Target exon mappings)
  - ambiguos (having more than one possible chromosome) assigned to **chrAmb**
  - unmapped sequences assigned to **chrUnmapped** (to distinguish from chrUn in target genome)

- gff file with the locations of the input sequences (gene)
  and remapped contents of the input gff files

Algorithm

- go through the input gff files, construct a dictionary {read_name -> {chr -> location}}
- add the lowest coordinate found on a given chromosome (overwriting previous values)
- sort the list with single candidate locations with 'chromocompare'

Author: Libor Morkovsky, 2012
"""

# This file is a part of Scrimer.
# See LICENSE.txt for details on licensing.
#    Copyright (C) 2012, 2013 Libor Morkovsky

import sys
import pybedtools

# reuse the generator fasta reader from lastz
from fasta_fragments import fasta_sequences

def get_overlap(a, b):
  return max(0, min(a[1], b[1]) - max(a[0], b[0]))
  
def merge_overlapping(ranges):
  """
  for a list of intervals stored as tuples
  return a list of intervals where 
  the overlapping regions are merged
  """
  ranges.sort()
  newranges = []
  idx = 0
  # go through elements
  while idx < len(ranges):
    # current region
    (start, end) = ranges[idx]
    
    # go through all regions starting after this one
    for nidx in xrange(idx + 1, len(ranges)):
      (nstart, nend) = ranges[nidx]
      # if there is an overlap or touch
      # try to extend current region
      if nstart <= end:
        end = max(end, nend)
        # continue with merging after this element
        idx = nidx
      # we're in sorted list, there will be no more overlaps
      else:
        break
    
    idx += 1
    newranges.append((start, end))
  
  return newranges
  
def coverage_by_target(contig, features, type="exon"):
  """
  for all features of given type
  sum the coverage with respect to the
  Target attribute
  return a dictionary {target_id: fraction_of_contig_covered}
  """
  # harvest the regions
  regions = dict()
  result = dict()
  for gffeature in features:
    if gffeature.fields[2] != 'exon' or 'Target' not in gffeature.attrs: continue

    # extract target info from the attributes
    target_id = gffeature.attrs['Target'].split()[0]
    
    # add to regions
    range_list = regions.setdefault(target_id, [])
    range_list.append( (int(gffeature.start), int(gffeature.end)) )
    
  # merge overlapping regions 
  for seq, ranges in regions.iteritems():
    newranges = merge_overlapping(ranges)
    
    # calculate the coverage
    covered_bases = sum(map(lambda x: get_overlap((contig.start, contig.end), x), newranges))
  
    result[seq] = float(covered_bases) / (contig.end - contig.start + 1)
    
  return result
    
  
def fasta_write(file, seqlist, line_len=70):
  for name, seq in seqlist:
    file.write(">%s\n" % name)
    for i in xrange(0, len(seq), line_len):
      file.write("%s\n" % seq[i : i + line_len])

import re
num = re.compile('[0-9]+')
def chromocompare(x, y):
  # compare chromosome ids
  # first by extracted numbers
  # then by length of the name
  # chr1 < chr1A < chrUn < chrUnV < chrZ

  # extract any number from the chromosome name
  xmatch = num.search(x)
  ymatch = num.search(y)
  
  # case with one or no chromosome number in the comparison
  if not all([xmatch, ymatch]):
    if any([xmatch, ymatch]):
      # None < anything, we need it >, return a negation of the comparison
      return -cmp(xmatch, ymatch)
    else:
      return cmp(x, y)
  
  xnum = int(xmatch.group())
  ynum = int(ymatch.group())
  if cmp(xnum, ynum) == 0:
    return cmp(len(x), len(y))
  else:
    return cmp(xnum, ynum)

def main():
  if len(sys.argv) < 2:
    sys.exit('use: %s contigs.fasta gff_with_mapped_exons [gff_with_mapped_exons ...] output.fasta remapped.gff' % sys.argv[0])
  
  # load the sequences into dict {read_name -> sequence}
  print >> sys.stderr, "loading sequences from %s" % sys.argv[1]
  seqfile = open(sys.argv[1])
  reads = { read_name : seq for (read_name, seq) in fasta_sequences(seqfile) }
  seqfile.close()
  
  # construct {read_name -> {chr -> location}}
  readloc = dict()
  features = dict()
  gffiles = [pybedtools.BedTool(fname) for fname in sys.argv[2:-2]]
  for gff in gffiles:
    #TODO: get rid of this requirement
    if gff.file_type != 'gff':
      sys.exit('only gff supported: %s' % repr(gff))

    print >> sys.stderr, "parsing gff %s" % repr(gff)

    for gffeature in gff:
      # add the feature to the dict for remapping later
      flist = features.setdefault(gffeature.chrom, [])
      flist.append(gffeature)
      
      # consider only exons with Target attribute
      if gffeature.fields[2] != 'exon' or 'Target' not in gffeature.attrs: continue

      # extract target info  from the attributes
      target_id, target_start, target_end, target_strand = gffeature.attrs['Target'].split()
      
      # get or create locations dict for current read
      locs = readloc.setdefault(gffeature.chrom, dict())
      # get or create candidate location on target chromosome
      pos = locs.setdefault(target_id, int(target_start))
      if target_start < pos:
        locs[target_id] = int(target_start)

  # construct inverted map-list {chr -> [(location, read_name)]}
  invmap = dict()
  ambiguous = set()
  for read_name, locations in readloc.iteritems():
    if len(locations) > 1:
      ambiguous.add(read_name)
    else:
      seq, pos = locations.iteritems().next()
      read_list = invmap.setdefault(seq, [])
      read_list.append( (pos, read_name) )
    
  chromosomes = invmap.keys()
  chromosomes.sort(cmp=chromocompare)

  # try to resolve the ambiguity by exon coverage
  print >> sys.stderr, "trying to resolve %d ambiguities " % len(ambiguous),
  amb_resolved = set()
  for read_name in ambiguous:
    coverages = [(coverage, target) for target, coverage in coverage_by_target(pybedtools.Interval(read_name, 0, len(reads[read_name])), features[read_name]).iteritems()]
    coverages.sort(reverse=True)

    # get an idea about the disambiguation process
    # print "disamb:", read_name, coverages
    
    # do the decisions only for decently covered contigs, leave others as amb
    if coverages[0][0] < 0.5: continue
    
    # if the best hit covers 4 times more than the second one, use it
    if coverages[0][0] < 4 * coverages[1][0] : continue

    # append the read to the chosen chromosome, 
    # as the others
    pos = readloc[read_name][coverages[0][1]]
    read_list = invmap.setdefault(coverages[0][1], [])
    read_list.append( (pos, read_name) )
    
    amb_resolved.add(read_name)

  print >> sys.stderr, "%d resolved" % len(amb_resolved)
  
  # ambiguously assigned reads, assign position 0 to all
  ambiguous -= amb_resolved
  invmap['chrAmb'] = [(0, read_name) for read_name in ambiguous]
  chromosomes.append('chrAmb')
  
  # unassigned reads
  all_reads = set(reads.iterkeys())
  assigned_reads = set(read_name for chrom, read_list in invmap.iteritems() for pos, read_name in read_list)
  unassigned_reads = all_reads - assigned_reads
  invmap['chrUnmapped'] = [(0, read_name) for read_name in unassigned_reads]
  chromosomes.append('chrUnmapped')

  # for the sorted list of chromosomes sort the positions and output the reads
  print >> sys.stderr, "writing output to %s and %s" % (sys.argv[-2], sys.argv[-1])
  splitlen = 150
  splitter = splitlen * 'N'

  fo_gff = open(sys.argv[-1], "w")
  fo_fasta = open(sys.argv[-2], "w")

  for chr in chromosomes:
    # sort the positions
    invmap[chr].sort()
    
    # construct the chromosome sequence
    chromseq = splitter.join(reads[read_name] for pos, read_name in invmap[chr])

    # output it as fasta
    fasta_write(fo_fasta, [(chr, chromseq)])
    
    # output a gff mRNA feature for each contig put on the chromosome
    # and remap all gff features from the input for reads at current chr
    feature_lengths = map(len, (reads[read_name] for pos, read_name in invmap[chr]))
    feature_positions = map(lambda x: sum(feature_lengths[:x]) + splitlen * x, xrange(0, len(feature_lengths)))
    for (index, (pos, read_name)) in enumerate(invmap[chr]):
      
      at = pybedtools.Attributes(' ');
      at['ID'] = read_name
      flist = [chr, 'virtual_genome', 'mRNA',
               str(feature_positions[index] + 1), str(feature_positions[index] + feature_lengths[index]),
               '1', '+', '.', str(at)]
      
      fo_gff.write(str(pybedtools.create_interval_from_list(flist)))
      
      # remap all the other features
      if read_name not in features: continue
      for feature in features[read_name]:
        # feature.attrs['Parent'] = read_name
        feature.chrom = chr
        feature.start += feature_positions[index]
        feature.end += feature_positions[index]
        fo_gff.write(str(feature))

  fo_gff.close()
  fo_fasta.close()
 
  
if __name__ == "__main__": main()



#!/usr/bin/env python

"""Given pairs of 'almost identical' sequences create clusters of sequences.
From each cluster select the longest sequence. Output names of all other 
sequences (for example to remove them from the data afterwards).

Uses disjoint sets forest to store the clusters so it should scale to millions of sequences.

Input

- custom formated (``--format=general:name1,size1,start1,name2,size2,start2,strand2,identity,coverage``)
  output from ``lastz`` on standard input

Output 

- names of sequences that were selected as reduntant
"""

# This file is a part of Scrimer.
# See LICENSE.txt for details on licensing.
#    Copyright (C) 2012, 2013 Libor Morkovsky


import sys

class DisjointSet:
    """implementation by John Machin found at
    http://stackoverflow.com/questions/3067529/looking-for-a-set-union-find-algorithm
    """
    def __init__(self):
        self.leader = {} # maps a member to the group's leader
        self.group = {} # maps a group leader to the group (which is a set)

    def add(self, a, b):
        leadera = self.leader.get(a)
        leaderb = self.leader.get(b)
        if leadera is not None:
            if leaderb is not None:
                if leadera == leaderb: return # nothing to do
                groupa = self.group[leadera]
                groupb = self.group[leaderb]
                if len(groupa) < len(groupb):
                    a, leadera, groupa, b, leaderb, groupb = b, leaderb, groupb, a, leadera, groupa
                groupa |= groupb
                del self.group[leaderb]
                for k in groupb:
                    self.leader[k] = leadera
            else:
                self.group[leadera].add(b)
                self.leader[b] = leadera
        else:
            if leaderb is not None:
                self.group[leaderb].add(a)
                self.leader[a] = leaderb
            else:
                self.leader[a] = self.leader[b] = a
                self.group[a] = set([a, b])

def main():
    ds = DisjointSet()
    seqlens = {}
    
    # create clusters
    for line in sys.stdin:
        fields = line.split()
        
        # count on seqlen being always the same for same seq id
        def addlen(id, idlen):
            if id not in seqlens:
                seqlens[id] = int(idlen)
        
        addlen(fields[0], fields[1])
        addlen(fields[3], fields[4])
        
        ds.add(fields[0], fields[3])

    # find the longest sequence 
    # (for each item in each group find its length, store it to a sorted list)
    clusters = [sorted((seqlens[id], id) for id in g) for g in ds.group.itervalues()]

    # from pprint import pprint
    # pprint(clusters)

    # output all but the biggest sequence id
    for cluster in clusters:
        for _, id in cluster[:-1]:
            print id
    
    print >> sys.stderr, len(seqlens), "sequences in", len(clusters), "clusters"

if __name__ == "__main__": main() 